"""Top-level package for aimet-ml."""
import static_ffmpeg

from . import features, processing, utils

static_ffmpeg.add_paths()

__author__ = """Pasit Jakkrawankul"""
__email__ = 'pasit.j@aimet.tech'
__version__ = '0.4.0'
